/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MDBMessageListenerEndPoint.java 5401 2010-03-05 09:54:09Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.container.mdb;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.jms.Message;
import javax.jms.MessageListener;

import org.ow2.easybeans.api.bean.EasyBeansMDB;
import org.ow2.easybeans.api.event.bean.EZBEventBeanInvocation;
import org.ow2.easybeans.event.bean.EventBeanInvocationEnd;
import org.ow2.easybeans.event.bean.EventBeanInvocationError;
import org.ow2.util.event.api.IEventDispatcher;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * This message listener end point is used for MDB message which use JMS as
 * message listener interface.
 * @author Florent Benoit
 */
public class MDBMessageListenerEndPoint extends MDBMessageEndPoint implements MessageListener {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(MDBMessageListenerEndPoint.class);

    /**
     * Use reflection or not ?
     */
    private boolean useReflection = false;

    /**
     * onMessage() method.
     */
    private Method onMessageMethod = null;

    /**
     * Event dispatcher.
     */
    private IEventDispatcher eventDispatcher = null;

    /**
     * Mehod event provider ID.
     */
    private String methodEventProviderId = "";

    /**
     * Constructor : Build an endpoint with a reference to the message end point
     * factory.
     * @param mdbMessageEndPointFactory the message end point factory.
     * @param easyBeansMDB the message driven bean object that is wrapped.
     */
    public MDBMessageListenerEndPoint(final MDBMessageEndPointFactory mdbMessageEndPointFactory,
            final EasyBeansMDB easyBeansMDB) {
        super(mdbMessageEndPointFactory, easyBeansMDB);
        this.eventDispatcher = ((MDBMessageEndPointFactory) getEasyBeansFactory()).getEventDispatcher();
        String j2EEManagedObjectId = ((MDBMessageEndPointFactory) getEasyBeansFactory()).getJ2EEManagedObjectId();
        this.methodEventProviderId = j2EEManagedObjectId + "/onMessage_javax.jms.Message";
        // if instance is not implementing the interface, use reflection for message delivery.
        if (!(easyBeansMDB instanceof MessageListener)) {
            logger.debug("Use reflection for message delivery as instance {0} is not implementing the interface", easyBeansMDB);
            this.useReflection = true;
            // get method
            try {
                this.onMessageMethod = easyBeansMDB.getClass().getMethod("onMessage", new Class[] {Message.class});
            } catch (SecurityException e) {
                throw new IllegalArgumentException("Cannot get the onMessage method on instance '" + easyBeansMDB + "'.", e);
            } catch (NoSuchMethodException e) {
                logger.error("No onMessage method found on bean named {0}", easyBeansMDB);
                throw new IllegalArgumentException("Cannot get the onMessage method on instance '" + easyBeansMDB + "'.", e);
            }
        }
    }

    /**
     * Passes a message to the listener.
     * @param message - the message passed to the listener
     */
    public void onMessage(final Message message) {
        // set classloader to EJB classloader
        ClassLoader oldCL = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(getEasyBeansFactory().getContainer().getClassLoader());

        // Create the bean invocation begin event.
        EZBEventBeanInvocation event = getMDBMessageEndPointFactory().getInvocationEventBegin(this.methodEventProviderId,
                new Object[] {message});
        long number = event.getInvocationNumber();
        this.eventDispatcher.dispatch(event);
        try {
            if (!this.useReflection) {
                ((MessageListener) getEasyBeansMDB()).onMessage(message);
                event = new EventBeanInvocationEnd(this.methodEventProviderId, number, null);
            } else {
                // invoke by using reflection
                try {
                    this.onMessageMethod.invoke(getEasyBeansMDB(), message);
                    event = new EventBeanInvocationEnd(this.methodEventProviderId, number, null);
                } catch (IllegalArgumentException e) {
                    event = new EventBeanInvocationError(this.methodEventProviderId, number, e);
                    throw new IllegalStateException("Cannot deliver the message", e);
                } catch (IllegalAccessException e) {
                    event = new EventBeanInvocationError(this.methodEventProviderId, number, e);
                    throw new IllegalStateException("Cannot deliver the message", e);
                } catch (InvocationTargetException e) {
                    event = new EventBeanInvocationError(this.methodEventProviderId, number, e);
                    logger.error("Cannot deliver the message", e.getTargetException());
                    throw new IllegalStateException("Cannot deliver the message", e.getTargetException());
                }
            }
        } finally {
            Thread.currentThread().setContextClassLoader(oldCL);
            this.eventDispatcher.dispatch(event);
        }
    }

}
