/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Serialization.java 5369 2010-02-24 14:58:19Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.util.marshalling;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;

/**
 * Allow to get the bytes of an object or to recreate an object from bytes.<br>
 * It also used a custom reader when recreating object as it uses the context
 * classloader in order to have an object with the right classloader.
 * @author Florent Benoit
 */
public final class Serialization {

    /**
     * Utility class, no public constructor.
     */
    private Serialization() {

    }


    /**
     * Gets an array of bytes corresponding to the given object.
     * @param object the object to serialize
     * @return an array of bytes.
     * @throws IOException if the object can't be turned into an array of bytes.
     */
    public static byte[] storeObject(final Serializable object) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        ObjectOutputStream oos  =  null;
        try {
            oos = new ObjectOutputStream(baos);
            oos.writeObject(object);
            oos.flush();
            return baos.toByteArray();
        } finally {
            if (oos != null)  {
                oos.close();
            }
            if (baos != null) {
                baos.close();
            }
        }
    }



    /**
     * Transforms the given array of bytes into an object.
     * @param bytes given array of bytes representing the object
     * @return object which has been unmarshalled.
     * @throws IOException if marshalling fails
     * @throws ClassNotFoundException if the class required for loading the object is not found.
     */
    public static Object loadObject(final byte[] bytes)
            throws IOException, ClassNotFoundException {
        // Do nothing if there aren't any bytes.
        if (bytes == null) {
            return null;
        }

        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
        CtxClassLoaderObjectInputStream ctxClassLoaderObjectInputStream = new CtxClassLoaderObjectInputStream(bais);
        try {
            return ctxClassLoaderObjectInputStream.readObject();
        } finally {
            if (ctxClassLoaderObjectInputStream != null) {
                ctxClassLoaderObjectInputStream.close();
            }
            if (ctxClassLoaderObjectInputStream != null) {
                bais.close();
            }

        }

    }






}
